from django.contrib.contenttypes.models import ContentType
from .models import Log
from . import _local


def model_to_dict(instance):
    opts = instance._meta

    ignore_fields = getattr(instance, 'LOGGING_IGNORE_FIELDS', [])
    only_fields = getattr(instance, 'LOGGING_ONLY_FIELDS', [])

    if only_fields:
        field_names = [f.attname for f in opts.fields if f.name in only_fields]
    elif ignore_fields:
        field_names = [f.attname for f in opts.fields if f.name not in ignore_fields]
    else:
        field_names = [f.attname for f in opts.fields]

    data = {f: getattr(instance, f, None) for f in field_names}

    return data


def diff(obj):
    d1 = obj.__attrs
    d2 = model_to_dict(obj)
    diffs = [(k, d2[k]) for k, v in d1.items() if v != d2[k]]
    return dict(diffs)


def create_log(obj, action, db, data=None):

    user = _local.request.user.id if _local.request else "System User"

    log_data = {
        'object_id': obj.pk,
        'model': ContentType.objects.get_for_model(obj._meta.model).model,
        'action': action,
        'data': data,
        'user': user
    }

    Log.objects.using(db).create(**log_data)




from threading import local

default_app_config = 'model_log.apps.ModelLogConfig'


class _Local(local):
    """
    Class for storing the request as local variable per thread
    """
    def __init__(self):
        self.request = None


_local = _Local()



from django.db.models.signals import post_init, post_save, post_delete
from django.apps.registry import apps

from .settings import MODELS_FOR_LOGGING, MODELS_FOR_EXCLUDE
from .signals import save_object, init_object_attrs, delete_object


def register_signals():

    if MODELS_FOR_LOGGING:
        registered_models = []
        for app in MODELS_FOR_LOGGING:
            item = app.split('.')
            if apps.all_models.get(item[-1]):  # if this is an app
                for v in apps.get_app_config(item[-1]).models.values():
                    if '{}.{}'.format(app, v.__name__) not in MODELS_FOR_EXCLUDE:
                        registered_models.append(v)
            else:   # if this is an model
                registered_models.append(apps.get_registered_model(item[-2], item[-1]))

        for model in registered_models:
            post_init.connect(init_object_attrs, sender=model)
            post_save.connect(save_object, sender=model)
            post_delete.connect(delete_object, sender=model)




try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object

from . import _local


class ModelLogMiddleware(MiddlewareMixin):
    """Expose request to the local variable.
        This middleware sets request as a local thread variable, making it
        available to the methods to allow tracking of the authenticated user
        making a change.
    """

    def process_request(self, request):
        _local.request = request

    def process_response(self, request, response):
        if hasattr(_local, "request"):
            del _local.request
        return response



from setuptools import setup

setup(
  name = 'django-model-logs',        
  packages = ['model_log'],  
  version = '0.1.4',     
  description = 'Library for logging changes of each model',  
  long_description=open('README.rst').read(),
  long_description_content_type='text/x-rst',
  author = "Juan Benitez",
  license="MIT",        
  author_email = 'juanbenitezdev@gmail.com',     
  url = 'https://github.com/JuanBenitezDev/django-model-log',   
  install_requires=['Django>=2.2'],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',     
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',
  ],
)